@@ -7,7 +7,7 @@
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation, either version 3 of the License, or
+#   the Free Software Foundation, either version 2 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -23,13 +23,13 @@
 
 `re.irts` <-
 function(x,...) {
-  if(!require('tseries', quietly=TRUE))
+  if(!requireNamespace('tseries', quietly=TRUE))
     irts <- function(...) message("package 'tseries' is required for re.irts")
 
   indexClass(x) <- "POSIXct"
   xx <- coredata(x)
 #  rownames(xx) <- attr(x,'irts.rownames')
-  irts(index(x),xx)
+  tseries::irts(index(x),xx)
 }
 
 `as.xts.irts` <-
