@@ -45,7 +45,19 @@ merge.xts <- function(...,
   if(is.null(suffixes)) {
     syms <- names(dots)
     syms[nchar(syms)==0] <- as.character(dots)[nchar(syms)==0]
-    if(is.null(syms)) syms <- as.character(dots)
+    if(is.null(syms)) {
+      # Based on makeNames() in merge.zoo()
+      syms <- substitute(alist(...))[-1L]
+      nm <- names(syms)
+      fixup <- if (is.null(nm)) seq_along(syms) else !nzchar(nm)
+      dep <- sapply(syms[fixup], function(x) deparse(x, nlines = 1L))
+      if(is.null(nm)) {
+        nm <- dep
+      } else if(any(fixup)) {
+        nm[fixup] <- dep
+      }
+      syms <- nm
+    }
   } else
   if(length(suffixes) != length(dots)) {
     warning("length of suffixes and does not match number of merged objects")
@@ -56,8 +68,6 @@ merge.xts <- function(...,
   }
 
   .times <- .External('number_of_cols', ..., PACKAGE="xts")
-  if(all(.times==0))
-    return(xts())
   symnames <- rep(syms, .times)  # moved call to make.names inside of mergeXts/do_merge_xts
 
   if(length(dots) == 1) {
