@@ -192,6 +192,8 @@ function(x) {
   if(is.xts(x)) {
     # if x is xts, drop HHMMSS from index
     if(any(indexClass(x)=='POSIXt')) {
+      # convert index to Date
+      index(x) <- as.Date(as.POSIXlt(index(x)))
       indexClass(x) <- "Date"  # set indexClass to Date
     }
     if(any(indexClass(x) %in% .classesWithoutTZ)) {
