@@ -114,6 +114,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   if( LENGTH(x)==0 || INTEGER(retside)[0]==0 ) {
     nrx = nrows(xindex);
     ncx = 0;
+    PROTECT(x = coerceVector(x, TYPEOF(y))); p++;
   }
   
   nry = nrows(y);
@@ -122,6 +123,7 @@ SEXP do_merge_xts (SEXP x, SEXP y,
   if( LENGTH(y)==0 || INTEGER(retside)[1]==0) {
     nry = nrows(yindex);
     ncy = 0;
+    PROTECT(y = coerceVector(y, TYPEOF(x))); p++;
   }
 
   len = nrx + nry;
@@ -1023,14 +1025,27 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   int coerce_to_double=0;
   if(args != R_NilValue) type_of = TYPEOF(CAR(args));
   while(args != R_NilValue) {
-    if( length(CAR(args)) > 0 )
+    if(length(CAR(args)) > 0) {
       ncs += ncols(CAR(args));
-    if(TYPEOF(CAR(args)) != type_of)
-      coerce_to_double = 1;  /* need to convert all objects if one needs to be converted */
+      /* need to convert all objects if one non-zero-width needs to be converted */
+      if(TYPEOF(CAR(args)) != type_of) {
+        coerce_to_double = 1;
+      }
+    }
     args = CDR(args);
     n++;
   }
 
+  /* return empty xts if all objects have no columns */
+  if(ncs < 1) {
+    SEXP s, t;
+    PROTECT(s = t = allocList(1)); P++;
+    SET_TYPEOF(s, LANGSXP);
+    SETCAR(t, install("xts"));
+    SEXP out = PROTECT(eval(s, env)); P++;
+    UNPROTECT(P);
+    return out;
+  }
 
   /* build an index to be used in all subsequent calls */
   args = argstart;
@@ -1109,28 +1124,16 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
   
     args = argstart; // reset args
     int ii, jj, iijj, jj_result;
-    int *int_result=NULL, *int_xtmp=NULL;
-    double *real_result=NULL, *real_xtmp=NULL;
 
     PROTECT(result = allocVector(TYPEOF(_INDEX), index_len * ncs)); P++;
-    switch(TYPEOF(result)) {
-      case LGLSXP:
-      case INTSXP:
-        int_result = INTEGER(result);
-        break;
-      case REALSXP:
-        real_result = REAL(result);
-        break;
-      default:
-        error("unsupported data type");
-    }
 
     SEXP ColNames, NewColNames;
     PROTECT(NewColNames = allocVector(STRSXP, ncs)); P++;
     ncs = 0;
     // REPROTECT xtmp inside for loop
-    PROTECT_INDEX idxtmp;
+    PROTECT_INDEX idxtmp, cnmtmp;
     PROTECT_WITH_INDEX(xtmp = NULL, &idxtmp); P++;
+    PROTECT_WITH_INDEX(ColNames = NULL, &cnmtmp); P++;
 
     for(i = 0, nc=0; args != R_NilValue; i = i+nc, args = CDR(args)) { // merge each object with index
       // i is object current being merged/copied
@@ -1154,40 +1157,91 @@ SEXP mergeXts (SEXP args) // mergeXts {{{
       nr = nrows(xtmp);
       nc = (0 == nr) ? 0 : ncols(xtmp);  // ncols(numeric(0)) == 1
       ncs += nc;
-      PROTECT(ColNames = getAttrib(CAR(args),R_DimNamesSymbol)); P++;
+
+      /* Use colnames from merged object, if it has them. Otherwise, use
+       * use deparsed names */
+      REPROTECT(ColNames = getAttrib(CAR(args),R_DimNamesSymbol), cnmtmp);
+      SEXP colnames = R_NilValue;
+      if(R_NilValue != ColNames) {
+        colnames = VECTOR_ELT(ColNames, 1);
+      }
+      if(R_NilValue == colnames) {
+        for(jj=0; jj < nc; jj++) {
+          SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
+        }
+      } else {
+        for(jj=0; jj < nc; jj++) {
+          SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(colnames,  jj));
+        }
+      }
+
       switch(TYPEOF(xtmp)) { // by type, insert merged data into result object
         case LGLSXP:
-        case INTSXP:
-          int_xtmp = INTEGER(xtmp);
-          for(jj=0; jj < nc; jj++) {
-            if(!isNull(ColNames) && !isNull(VECTOR_ELT(ColNames,1))) {
-              /* if merged object has colnames, use these, otherwise use deparse names */
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(VECTOR_ELT(ColNames,1),jj));
-            } else {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
+          {
+            int *xtmp_ = LOGICAL(xtmp);
+            int *result_ = LOGICAL(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
-            for(ii=0; ii < nr; ii++) {
-              iijj = ii + jj * nr;
-              jj_result = ii + ( (i+jj) * nr);
-              int_result[ jj_result ] = int_xtmp[ iijj ];
+          }
+          break;
+        case INTSXP:
+          {
+            int *xtmp_ = INTEGER(xtmp);
+            int *result_ = INTEGER(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
           }
           break;
         case REALSXP:
-          real_xtmp = REAL(xtmp);
-          for(jj=0; jj < nc; jj++) {
-            if(!isNull(ColNames) && !isNull(VECTOR_ELT(ColNames,1))) {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(VECTOR_ELT(ColNames,1),jj));
-            } else {
-              SET_STRING_ELT(NewColNames, i+jj, STRING_ELT(symnames,i+jj));
-            }
-            for(ii=0; ii < nr; ii++) {
-              iijj = ii + jj * nr;
-              jj_result = ii + ( (i+jj) * nr);
-              real_result[ jj_result ] = real_xtmp[ iijj ];
+          {
+            double *xtmp_ = REAL(xtmp);
+            double *result_ = REAL(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
             }
           }
           break;
+        case CPLXSXP:
+          {
+            Rcomplex *xtmp_ = COMPLEX(xtmp);
+            Rcomplex *result_ = COMPLEX(result);
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                result_[jj_result] = xtmp_[iijj];
+              }
+            }
+          }
+          break;
+        case STRSXP:
+          {
+            for(jj=0; jj < nc; jj++) {
+              for(ii=0; ii < nr; ii++) {
+                iijj = ii + jj * nr;
+                jj_result = ii + (i+jj) * nr;
+                SET_STRING_ELT(result, jj_result, STRING_ELT(xtmp, iijj));
+              }
+            }
+          }
+          break;
+        default:
+          error("unsupported data type");
+          break;
       }
     }
 
